%{
/* file: scan.l     
   Project: Phase II
   Note:  use of "y.tab.h" means you need to compile parse.y with -d flag
   Modified by: Nick Toothman 
   11/5/2009
   
*/ 

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "y.tab.h"
#include "symtab.h"

#ifndef TRUE
#define TRUE 1 
#endif

#ifndef FALSE 
#define FALSE 0 
#endif

#define MAXTOKENLEN 50 
#define MAX_MSG_LEN 50 

typedef struct {
  char text[256];
} WordType;

char tokenString[MAXTOKENLEN+1];
int line_no = 1;
int col_no = 1;
int c;
%}

/* define yytext to be an array rather than a pointer */
%array

digit       [0-9]
integer     {digit}+
float       {digit}+|{digit}+\.{digit}*
letter      [a-zA-Z]
identifier  ({letter}|_)+({digit}|{letter})*
string      \"[a-zA-Z0-9 \t\n\\\"\.\!\?\']+\"
newline     \n
nlineLit    \'\\n\'
charLiteral \'({letter}|{digit})\'
whitespace  [ \t]+
comment     (\/\/|\/\*)+.*(\*\/)*

%%

"if"            { return IF;}
"then"          { return THEN;}
"else"          { return ELSE;}
"end"           { return END;}
"repeat"        { return REPEAT;}
"until"         { return UNTIL;}
"read"          { return READ;}
"write"         { return WRITE;}
"while"		{ return WHILE; }
"break"		{ return BREAK; }
"return"	{ return RETURN; }
"class"		{ return CLASS; }
"new"		{ return NEW; }
"const"		{ return CONST; }
"void" 		{ return VOID; }

"<="		{ return LTE; }
"<"		{ return LT; }
">" 		{ return GT; }
">=" 		{ return GTE; }
"!=" 		{ return INEQ; }
"="             { return ASSIGN;}
"=="            { return EQ;}
"+"             { return PLUS;}
"-"             { return MINUS;}
"*"             { return TIMES;}
"/"             { return OVER;}
"%"		{ return MOD; }
","		{ return COMMA; }

"("             { return LPAREN; }
"{"		{ return LBRACE; }
"&&"		{ return AND; }
"||"		{ return OR; }
"++"		{ return INCR; }
"--"		{ return DECR; }


{charLiteral}   {  
                   return CHARLIT;
                }
{nlineLit}	{ return CHARLIT; }
")"             { 
                  return RPAREN;
                }
"}"		{ return RBRACE; }
";"             { return SEMI;}

{integer}       {
                  return NUM;
                }
{float}         { 
                  strncpy(tokenString,yytext,yyleng);
                  insert(tokenString, yyleng, line_no, 'F');
                  return NUM;
                }

{identifier}    { 
                  strncpy(tokenString,yytext,yyleng);
                  insert(tokenString, yyleng, line_no, 'N');
                  return ID;
                }
{string}	{
		  return STRING;
		}

{newline}       { line_no++; col_no = 1; }

{whitespace}    { col_no++; /* count columns then skip whitespace */}

{comment}	{ }

%%
